/* Example program to calculate a grid of points for a Gastner-Newman
 * cartogram using the cartogram.c code
 *
 * Written by Mark Newman
 *
 * See http://www.umich.edu/~mejn/ for further details.
 */


#include <stdio.h>
#include <stdlib.h>

#include "cart.h"


/* The parameter OFFSET specifies a small amount to be added the density in
 * every grid square, as a Fraction of the mean density on the whole
 * lattice.  This prevents negative densities from being generated by
 * numerical errors in the FFTs which can cause problems for the
 * integrator.  If the program is giving weird behavior, particularly at
 * the beginning of a calculation, try increasing this quantity by a factor
 * of 10.
 */

#define OFFSET 0.005


/* Function to read population data into the array rho.  Returns 1 if there
 * was a problem, zero otherwise */

int readpop(FILE *stream, double **rho, int xsize, int ysize)
{
  int ix,iy;
  int n;
  double mean;
  double sum=0.0;

  for (iy=0; iy<ysize; iy++) {
    for (ix=0; ix<xsize; ix++) {
      n = fscanf(stream,"%lf",&rho[ix][iy]);
      if (n!=1) return 1;
      sum += rho[ix][iy];
    }
  }

  mean = sum/(xsize*ysize);
  for (iy=0; iy<ysize; iy++) {
    for (ix=0; ix<xsize; ix++) {
      rho[ix][iy] += OFFSET*mean;
    }
  }

  return 0;
}


/* Function to make the grid of points */

void creategrid(double *gridx, double *gridy, int xsize, int ysize)
{
  int ix,iy;
  int i;

  for (iy=0,i=0; iy<=ysize; iy++) {
    for (ix=0; ix<=xsize; ix++) {
      gridx[i] = ix;
      gridy[i] = iy;
      i++;
    }
  }
}


/* Function to write out the grid points */

void writepoints(FILE *stream, double *gridx, double *gridy, int npoints)
{
  int i;

  for (i=0; i<npoints; i++) fprintf(stream,"%g %g\n",gridx[i],gridy[i]);
}


main(int argc, char *argv[])
{
  int xsize,ysize;
  double *gridx,*gridy;  // Array for grid points
  double **rho;          // Initial population density
  FILE *infp;
  FILE *outfp;

  /* Read the command-line parameters and open the input and output files */

  if (argc<5) {
    fprintf(stderr,"Usage: %s xsize ysize inputfile outputfile\n",argv[0]);
    exit(1);
  }
  xsize = atoi(argv[1]);
  if (xsize<1) {
    fprintf(stderr,"%s: xsize botched\n",argv[0]);
    exit(2);
  }
  ysize = atoi(argv[2]);
  if (ysize<1) {
    fprintf(stderr,"%s: ysize botched\n",argv[0]);
    exit(3);
  }
  infp = fopen(argv[3],"r");
  if (infp==NULL) {
    fprintf(stderr,"%s: unable to open file `%s'\n",argv[0],argv[3]);
    exit(4);
  }
  outfp = fopen(argv[4],"w");
  if (outfp==NULL) {
    fprintf(stderr,"%s: unable to open file `%s'\n",argv[0],argv[4]);
    exit(5);
  }

  /* Allocate space for the cartogram code to use */

  cart_makews(xsize,ysize);

  /* Read in the population data, transform it, then destroy it again */

  rho = cart_dmalloc(xsize,ysize);
  if (readpop(infp,rho,xsize,ysize)) {
    fprintf(stderr,"%s: density file contains too few or incorrect data\n",
	    argv[0]);
    exit(6);
  }
  cart_transform(rho,xsize,ysize);
  cart_dfree(rho);

  /* Create the grid of points */

  gridx = malloc((xsize+1)*(ysize+1)*sizeof(double));
  gridy = malloc((xsize+1)*(ysize+1)*sizeof(double));
  creategrid(gridx,gridy,xsize,ysize);

  /* Make the cartogram */

  cart_makecart(gridx,gridy,(xsize+1)*(ysize+1),xsize,ysize,0.0);

  /* Write out the final positions of the grid points */

  writepoints(outfp,gridx,gridy,(xsize+1)*(ysize+1));

  /* Free up the allocated space */

  cart_freews(xsize,ysize);
  free(gridx);
  free(gridy);

  /* Close the input and output files */

  fclose(infp);
  fclose(outfp);
}
